<?php

namespace Drupal\users_account_modal;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\users_account_modal\Event\PagesEvent;
use Drupal\users_account_modal\Event\UsersAccountModalEvents;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * A helper class for managing configured account pages.
 */
class AccountPageHelper {

  use StringTranslationTrait;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs an AccountPageHelper object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   */
  public function __construct(ModuleHandlerInterface $moduleHandler, EventDispatcherInterface $eventDispatcher, ConfigFactoryInterface $configFactory) {
    $this->moduleHandler = $moduleHandler;
    $this->eventDispatcher = $eventDispatcher;
    $this->configFactory = $configFactory;
  }

  /**
   * @return array
   */
  public function getPages() {
    $pages = [
      'page' => [
        'label' => $this->t('User page'),
        'routes' => ['user.page'],
        'form' => FALSE,
      ],
      'login' => [
        'label' => $this->t('Login'),
        'routes' => ['user.login'],
        'form' => 'user_login_form',
      ],
      'register' => [
        'label' => $this->t('Register'),
        'routes' => ['user.register'],
        'form' => 'user_register_form',
      ],
      'password' => [
        'label' => $this->t('Password reset'),
        'routes' => ['user.pass'],
        'form' => 'user_pass',
      ],
      'cancel' => [
        'label' => $this->t('Account cancellation'),
        'routes' => ['user.cancel_confirm'],
        'form' => 'user_cancel',
      ],
    ];

    if ($this->moduleHandler->moduleExists('profile')) {
      $pages['profile_add'] = [
        'label' => $this->t('Profile add'),
        'routes' => ['/entity\.profile\.type\..+.\.user_profile_form\.add/'],
        'form' => '/profile_.+_add_form/',
      ];

      $pages['profile_edit'] = [
        'label' => $this->t('Profile edit'),
        'routes' => ['entity.profile.edit_form'],
        'form' => '/profile_.+_edit_form/',
      ];
    }

    $event = new PagesEvent($pages);
    $this->eventDispatcher->dispatch($event, UsersAccountModalEvents::PAGES);
    return $event->getPages();
  }

  /**
   * @return array
   */
  public function getPageOptions() {
    $pages = $this->getPages();
    $options = [];

    foreach ($pages as $pageId => $page) {
      $options[$pageId] = $page['label'];
    }

    return $options;
  }

  /**
   * @param $page
   *
   * @return array
   */
  public function getRoutes($page) {
    $pages = $this->getPages();
    $routes = [];

    if (isset($pages[$page])) {
      $routes = $pages[$page]['routes'];
    }

    return $routes;
  }

  /**
   * @return array
   */
  public function getAllRoutes() {
    $pages = $this->getPages();
    $routes = [];

    foreach ($pages as $pageId => $page) {
      $routes += $page['routes'];
    }

    return $routes;
  }

  /**
   * @return array
   */
  public function getEnabledPages() {
    $config = $this->configFactory->get('users_account_modal.settings');
    $enabledPages = $config->get('enabled_pages');
    $pages = $this->getPages();
    $results = [];

    foreach ($pages as $pageId => $pageInfo) {
      if (in_array($pageId, $enabledPages, TRUE)) {
        $results[$pageId] = $pageInfo;
      }
    }

    return $results;
  }

  /**
   * @param $pageId
   *
   * @return mixed|null
   */
  public function getPage($pageId) {
    $pages = $this->getPages();

    if (!isset($pages[$pageId])) {
      return NULL;
    }

    return $pages[$pageId];
  }

  /**
   * @param $route
   *
   * @return int|null|string
   */
  public function getPageFromRoute($route) {
    $page = NULL;

    foreach ($this->getEnabledPages() as $pageId => $pageInfo) {
      if (in_array($route, $pageInfo['routes'], TRUE)) {
        $page = $pageId;

        break;
      }

      foreach ($pageInfo['routes'] as $pageRoute) {
        if (strpos($pageRoute, '/') === 0 && preg_match($pageRoute, $route)) {
          $page = $pageId;

          break;
        }
      }
    }

    return $page;
  }

  /**
   * @param $formId
   *
   * @return int|null|string
   */
  public function getPageFromFormId($formId) {
    $page = NULL;

    foreach ($this->getEnabledPages() as $pageId => $pageInfo) {
      if ($formId === $pageInfo['form']) {
        $page = $pageId;
        break;
      }

      if (strpos($pageInfo['form'], '/') === 0 && preg_match($pageInfo['form'], $formId)) {
        $page = $pageId;
        break;
      }
    }

    return $page;
  }

}
